
/*
 *	HART Common Practice 
 *	Rev 7 ddrev 2 using import of ddrev 1
 *
 */


#include "macros.ddl"
#include "bi_codes.h"

MANUFACTURER  STANDARD, DEVICE_TYPE _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 2
/* This file conforms to the document: 
 *    HART Smart Communications Protocol - Common Practice Command Specification, 
 *    Revision 7.0
 * This file conforms to the document:
 *    HART Smart Communications Protocol - Common Tables, 
 *    Revision 6.0 
 * This file conforms to the document:
 *    HART Smart Communications Protocol - Command Summary Information, 
 *    Revision 7.0
 */

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 1
{
	EVERYTHING;
	REDEFINITIONS
	{
		REDEFINE METHOD loop_test
		{
			LABEL [loop_test];

			HELP [loop_test_help];

			DEFINITION
			{
				char status[STATUS_SIZE];
				int choice;
				long var_id[1];
				int use_abort_method;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1=60;
				slen2=60;
				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);

				IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_RESPONSE_CODES();

				use_abort_method = 0;
				do {
					get_dictionary_string(choose_analog_output_level,disp_string1,slen1);
					get_dictionary_string(choices_4_20_other_end,disp_string2,slen2);
					choice = SELECT_FROM_LIST(disp_string1, disp_string2);
					switch ( choice ) {
					case 0:
						assign_float(PV.ANALOG_VALUE,4.0);
						break;

					case 1:
						assign_float(PV.ANALOG_VALUE,20.0);
						break;

					case 2:
						get_dictionary_string(loop_output,disp_string1,slen1);
						GET_DEV_VAR_VALUE(disp_string1, PV.ANALOG_VALUE);
						break;

					case 3:
					default:
						process_abort();
						break;
					}
					send(40,status);
					if (!use_abort_method) {
						if (( status[STATUS_RESPONSE_CODE] != 0)
								&& ( status[STATUS_RESPONSE_CODE] != 3)
								&& ( status[STATUS_RESPONSE_CODE] != 4)) {
							display_response_status(40,status[STATUS_RESPONSE_CODE]);
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
						if ( status[STATUS_RESPONSE_CODE] == 0) {
							remove_abort_method(return_to_normal);
							add_abort_method(leave_fixed_current_mode);
							use_abort_method =1;
						}
					}
					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(40,status[STATUS_RESPONSE_CODE]);
						if (( status[STATUS_RESPONSE_CODE] != 3)
								&& ( status[STATUS_RESPONSE_CODE] != 4)) {
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
					}
					else {
						var_id[0] = VARID(PV.ANALOG_VALUE);
						get_dictionary_string(fld_dev_otpt_fixed,disp_string1,slen1);
						acknowledge(disp_string1,var_id);
					}
				} while ( choice < 3);
				process_abort();
			}
		}

		REDEFINE METHOD dac_trim
		{
			LABEL [DAC_trim];

			HELP [digital_to_analog_trim_help];

		/*	APPHELP "This type of calibration is usually used to compensate for"
						  " drifting or aging output electronics";
		*/
			DEFINITION
			{
				int retry;
				char status[STATUS_SIZE];
				long var_ids[2];
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1=60;
				slen2=60;

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);

				get_dictionary_string(connect_reference_meter,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);

				IGNORE_ALL_DEVICE_STATUS();
				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_RESPONSE_CODES();

				get_dictionary_string(setting_fld_dev_4_ma,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				assign_float(PV.ANALOG_VALUE,4.0);
				send(40,status);

				if ( status[STATUS_RESPONSE_CODE] ) {
					display_response_status(40,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}

				remove_abort_method(return_to_normal);
				add_abort_method(leave_fixed_current_mode);

				var_ids[0] = VARID(PV.ANALOG_VALUE);

				do {
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_DEV_VAR_VALUE(disp_string1, PV.ANALOG_VALUE);
					send(45,status);
					assign_float(PV.ANALOG_VALUE,4.0);
					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(45,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if ( status[STATUS_RESPONSE_CODE]) {
						display_response_status(45,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(fld_dev_otpt_equal_ref_meter,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=select_from_list(disp_string1,var_ids,disp_string2);
					}
				} while (retry);

				get_dictionary_string(setting_fld_dev_20_ma,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				assign_float(PV.ANALOG_VALUE,20.0);
				send(40,status);

				if ( status[STATUS_RESPONSE_CODE]) {
					display_response_status(40,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}

				do {
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_DEV_VAR_VALUE(disp_string1, PV.ANALOG_VALUE);
					send(46,status);
					assign_float(PV.ANALOG_VALUE,20.0);

					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(46,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if ( status[STATUS_RESPONSE_CODE]) {
						display_response_status(46,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(fld_dev_otpt_equal_ref_meter,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=select_from_list(disp_string1,var_ids,disp_string2);
					}
				} while (retry);

				process_abort() ;
			}
		}

		REDEFINE METHOD scaled_dac_trim
		{
			LABEL [scaled_DAC_trim];

			HELP [digital_to_analog_scaled_trim_help];

		/*  APPHELP "This type of calibration is usually used to compensate for"
				  " drifting or aging output electronics. A common usage of this is when the"
				  " reference is in different engineering units than the Analog Output"
				  " (Ex. reference meter is 0-100% when the Field Device output is in mA";
		*/
			DEFINITION
			{
				int retry;
				char status[STATUS_SIZE];
				float low, high, tmp_av;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1 =60;
				slen2 =60;

				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();
				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();

				low = 4.0;
				high = 20.0;

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);

				do {
					get_dictionary_string(trim_scaled_from_low_to_high,disp_string1,slen1);
					get_dictionary_string(proceed_change,disp_string2,slen2);
					retry = SELECT_FROM_LIST(disp_string1,disp_string2);
					if (retry ) {
						do {
							get_dictionary_string(set_scale_lo_output_value,disp_string1,slen1);
							GET_LOCAL_VAR_VALUE(disp_string1, low);
							get_dictionary_string(set_scale_hi_output_value,disp_string1,slen1);
							GET_LOCAL_VAR_VALUE(disp_string1, high);
							if (!(high - low)) {
								get_dictionary_string(high_low_0_invalid,disp_string1,slen1);
								ACKNOWLEDGE(disp_string1);
							}
						} while (!(high - low));
					}
				} while (retry);

				get_dictionary_string(connect_reference_meter,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);

				get_dictionary_string(setting_fld_dev_4_ma,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				assign_float(PV.ANALOG_VALUE,4.0);
				send(40,status);
				if ( status[STATUS_RESPONSE_CODE] ) {
					display_response_status(40,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}

				remove_abort_method(return_to_normal);
				add_abort_method(leave_fixed_current_mode);

				do {
					tmp_av = low;
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_LOCAL_VAR_VALUE(disp_string1, tmp_av);
					assign_float(PV.ANALOG_VALUE, 16 / (high - low) * (tmp_av - low) + 4);
					send(45,status);
					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(45,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if ( status[STATUS_RESPONSE_CODE]) {
						display_response_status(45,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(scaled_otpt_low_eq_dev,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=SELECT_FROM_LIST(disp_string1,disp_string2);
					}
				} while (retry);

				get_dictionary_string(setting_fld_dev_20_ma,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				assign_float(PV.ANALOG_VALUE, 20.0);
				send(40,status);
				if ( status[STATUS_RESPONSE_CODE] ) {
					display_response_status(40,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}

				do {
					tmp_av = high;
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_LOCAL_VAR_VALUE(disp_string1, tmp_av);
					assign_float(PV.ANALOG_VALUE, 16 / (high - low) * (tmp_av - low) + 4);
					send(46,status);
					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(46,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if ( status[STATUS_RESPONSE_CODE]) {
						display_response_status(46,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(scaled_otpt_high_eq_dev,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=SELECT_FROM_LIST(disp_string1,disp_string2);
					}
				} while (retry);

				process_abort();
			}
		}

		REDEFINE METHOD zero_trim
		{
			LABEL [zero_trim];

			HELP [zero_trim_help];

			DEFINITION
			{
				char status[STATUS_SIZE];
				int retry;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1=60;
				slen2=60;

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);
				get_dictionary_string(this_will_affect_snsr_cal,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);

				do {
					get_dictionary_string(apply_0_input_to_snsr,disp_string1,slen1);
					ACKNOWLEDGE(disp_string1);

					get_dictionary_string(snsr_input_stabilizing,disp_string1,slen1);
					DELAY(3,disp_string1);

					send(43,status);

					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(43,status[STATUS_RESPONSE_CODE]);
						if ((status[STATUS_RESPONSE_CODE] != 9)
								&& ( status[STATUS_RESPONSE_CODE] != 10)) {
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
						get_dictionary_string(zero_the_input_and_try_again,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=SELECT_FROM_LIST(disp_string1,disp_string2);
					}
					else {
						get_dictionary_string(snsr_zero_succeeded,disp_string1,slen1);
						DELAY(3,disp_string1);
						process_abort();
					}
				} while ( !retry);
				process_abort();
			}
		}

		REDEFINE METHOD leave_fixed_current_mode
		{
			LABEL [leaving_fixed_current_mode];

			HELP [leaving_fixed_current_mode_help_2];

			DEFINITION
			{
				char status[STATUS_SIZE];
				long var_id[1];
				char disp_string1[60];
				int slen1;

				slen1 =60;

				assign_float(PV.ANALOG_VALUE,0.0);

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				do {
					get_dictionary_string(return_fld_dev_to_orig_otpt,disp_string1,slen1);
					DELAY(3,disp_string1);
					send(40,status);
					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(40,status[STATUS_RESPONSE_CODE]);
					}
				} while ( status[STATUS_RESPONSE_CODE]);
				get_dictionary_string(loop_may_be_returned_to_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
			}
		}


		REDEFINE METHOD applied_rerange
		{
			LABEL [apply_values];

			HELP [applied_values_help];

			DEFINITION
			{
				int rc;
				long varid[3];
				char status[STATUS_SIZE];
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1 =60;
				slen2 =60;

				IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);

				varid[0] =VARID(PV.DIGITAL_VALUE);
				varid[1] =VARID(PV.DIGITAL_UNITS);
				while ( 1 ) {
					get_dictionary_string(set_the,disp_string1,slen1);
					get_dictionary_string(choice_4_20_exit,disp_string2,slen2);
					switch(SELECT_FROM_LIST(disp_string1,disp_string2)) {
					case 0:
						get_dictionary_string(apply_new_4ma_input,disp_string1,slen1);
						ACKNOWLEDGE(disp_string1);
						do {
							send_command(1);
							get_dictionary_string(current_applied_process_val,disp_string1,slen1);
							get_dictionary_string(set_4ma_read_new_leave,disp_string2,slen2);
							rc = select_from_list(disp_string1,varid,disp_string2);
						} while(rc==1);
						if( rc == 0 ) {
							send(37,status);
							if ( status[STATUS_RESPONSE_CODE] ) {
								display_response_status(37,status[STATUS_RESPONSE_CODE]);
							}
							send_command(15);
						}
						break;
					case 1:
						get_dictionary_string(apply_new_20ma_input,disp_string1,slen1);
						ACKNOWLEDGE(disp_string1);
						do {
							send_command(1);
							get_dictionary_string(current_applied_process_val,disp_string1,slen1);
							get_dictionary_string(set_20ma_read_new_leave,disp_string2,slen2);
							rc = select_from_list(disp_string1,varid,disp_string2);
						} while(rc==1);
						if( rc == 0 ) {
							send(36,status);
							if ( status[STATUS_RESPONSE_CODE] ) {
								display_response_status(36,status[STATUS_RESPONSE_CODE]);
							}
							send_command(15);
						}
						break;
					case 2:
						process_abort();
					default:
						process_abort();
					}
				}
			}
		}

		REDEFINE METHOD fixed_analog_output
		{
			LABEL [fix_analog_output];

			HELP [fix_analog_output_help];

			DEFINITION
			{
				char status[STATUS_SIZE];
				int choice;
				long var_id[3];
				int use_abort_method;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1 =60;
				slen2 =60;

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				use_abort_method = 0;

				get_dictionary_string(select_analog_output_to_trim,disp_string1,slen1);
				GET_DEV_VAR_VALUE(disp_string1,analog_output_code);
				get_dictionary_string(select_analog_output_units,disp_string1,slen1);
				GET_DEV_VAR_VALUE(disp_string1,
						dynamic_variables[ivar_value(analog_output_code)].ANALOG_UNITS);
				var_id[0] = VARID(analog_output_code);
				var_id[2] = VARID(dynamic_variables[ivar_value(analog_output_code)].ANALOG_UNITS);
				do {
					if (ivar_value(dynamic_variables[ivar_value(analog_output_code)].ANALOG_UNITS)== 39) {
						get_dictionary_string(choose_analog_output_level,disp_string1,slen1);
						get_dictionary_string(choices_4_20_other_end,disp_string2,slen2);
						choice = SELECT_FROM_LIST(disp_string1, disp_string2);
						switch ( choice ) {
						case 0:
							assign_float(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,4.0);
							break;

						case 1:
							assign_float(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,20.0);
							break;

						case 2:
							get_dictionary_string(enter_desired_analog_output,disp_string1,slen1);
							GET_DEV_VAR_VALUE(disp_string1,
									dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);
							break;

						case 3:
						default:
							process_abort();
							break;
						}
					}
					else {
						get_dictionary_string(choose_analog_output_level,disp_string1,slen1);
						get_dictionary_string(new_value_end,disp_string2,slen2);
						choice = SELECT_FROM_LIST(disp_string1,disp_string2);
						switch(choice) {
						case 0:
							get_dictionary_string(enter_desired_analog_output,disp_string1,slen1);
							GET_DEV_VAR_VALUE(disp_string1,
									dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);
							break;
						case 1:
							process_abort();
							break;
						}
					}

					send(66,status);
					if ( !use_abort_method ) {
						if (( status[STATUS_RESPONSE_CODE] != 0)
								&& ( status[STATUS_RESPONSE_CODE] != 3)
								&& ( status[STATUS_RESPONSE_CODE] != 4)) {
							display_response_status(66,status[STATUS_RESPONSE_CODE]);
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
						if ( status[STATUS_RESPONSE_CODE] == 0) {
							use_abort_method = 1;
							remove_abort_method(return_to_normal);
							add_abort_method(leave_analog_fixed_current_mode);
						}
					}
					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(66,status[STATUS_RESPONSE_CODE]);
						if (( status[STATUS_RESPONSE_CODE] != 3)
								&& ( status[STATUS_RESPONSE_CODE] != 4)) {
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
					}
					else {
						var_id[1]=VARID(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);
						get_dictionary_string(fld_dev_AO_is_fixed_at,disp_string1,slen1);
						acknowledge(disp_string1,var_id);
					}
				} while ( choice < 3);
			}
		}

		REDEFINE METHOD analog_dac_trim
		{
			LABEL [trim_analog_output];

			HELP [digital_to_analog_trim_help];

		/*	APPHELP "This type of calibration is usually used to compensate for"
						  " drifting or aging output electronics";
		*/
			DEFINITION
			{
				char status[STATUS_SIZE];
				long var_id[3];
				int retry;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1 =60;
				slen2 =60;

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);
				get_dictionary_string(connect_reference_meter,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);

				get_dictionary_string(select_analog_output_to_trim,disp_string1,slen1);
				GET_DEV_VAR_VALUE(disp_string1,analog_output_code);
				get_dictionary_string(select_analog_output_units,disp_string1,slen1);
				GET_DEV_VAR_VALUE(disp_string1,
						dynamic_variables[ivar_value(analog_output_code)].ANALOG_UNITS);

				var_id[0] = VARID(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);
				var_id[1] = VARID(dynamic_variables[ivar_value(analog_output_code)].ANALOG_UNITS);

				assign_var(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,
						dynamic_variables[ivar_value(analog_output_code)].LOWER_ENDPOINT_VALUE);

				get_dictionary_string(about_to_set_fld_dev_output,disp_string1,slen1);
				acknowledge(disp_string1,var_id);
				send(66,status);

				if (status[STATUS_RESPONSE_CODE]) {
					display_response_status(66,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}
				else {
					remove_abort_method(return_to_normal);
					add_abort_method(leave_analog_fixed_current_mode);
				}

				do {
					assign_var(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,
							dynamic_variables[ivar_value(analog_output_code)].LOWER_ENDPOINT_VALUE);
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_DEV_VAR_VALUE(disp_string1,
							dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);
					send(67,status);

					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(67,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if ( status[STATUS_RESPONSE_CODE]) {
						display_response_status(67,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(fld_dev_output_eq_ref_meter,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=select_from_list(disp_string1,var_id,disp_string2);
					}
				} while (retry);

				assign_var(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,
						dynamic_variables[ivar_value(analog_output_code)].UPPER_ENDPOINT_VALUE);

				get_dictionary_string(about_to_set_fld_dev_output,disp_string1,slen1);
				acknowledge(disp_string1,var_id);
				send(66, status);

				if (status[STATUS_RESPONSE_CODE]) {
					display_response_status(66,status[STATUS_RESPONSE_CODE]);
					get_dictionary_string(abort_last_error,disp_string1,slen1);
					DELAY(2,disp_string1);
					process_abort();
				}

				do {
					assign_var(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,
							dynamic_variables[ivar_value(analog_output_code)].UPPER_ENDPOINT_VALUE);
					get_dictionary_string(enter_meter_value,disp_string1,slen1);
					GET_DEV_VAR_VALUE(disp_string1,
							dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE);

					send(68,status);
					if (( status[STATUS_RESPONSE_CODE] == 3 ) || ( status[STATUS_RESPONSE_CODE] == 4)) {
						display_response_status(68,status[STATUS_RESPONSE_CODE]);
						retry = 1;
					}
					else if (status[STATUS_RESPONSE_CODE]) {
						display_response_status(68,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(abort_last_error,disp_string1,slen1);
						DELAY(2,disp_string1);
						process_abort();
					}
					else {
						get_dictionary_string(fld_dev_output_eq_ref_meter,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=select_from_list(disp_string1,var_id,disp_string2);
					}
				} while (retry);
				process_abort();
			}
		}

		REDEFINE METHOD tv_zero_trim
		{
			LABEL [field_device_variable_zero_trim];

			HELP [zero_trim_help];

			DEFINITION
			{
				char status[STATUS_SIZE];
				int tv;
				int retry;
				char disp_string1[60];
				char disp_string2[60];
				int slen1;
				int slen2;

				slen1=60;
				slen2=60;

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				get_dictionary_string(remove_loop_from_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
				add_abort_method(return_to_normal);
				get_dictionary_string(this_will_affect_snsr_cal,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);

				get_dictionary_string(select_fld_dev_var_snsr_trim,disp_string1,slen1);
				GET_DEV_VAR_VALUE(disp_string1,transmitter_variable_code);

				do {
					get_dictionary_string(apply_0_input_to_snsr,disp_string1,slen1);
					ACKNOWLEDGE(disp_string1);

					get_dictionary_string(snsr_input_stabilizing,disp_string1,slen1);
					DELAY(3,disp_string1);

					send(52,status);

					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(52,status[STATUS_RESPONSE_CODE]);
						if ((status[STATUS_RESPONSE_CODE] != 2)
								&& ( status[STATUS_RESPONSE_CODE] != 9)
								&& ( status[STATUS_RESPONSE_CODE] != 10)) {
							get_dictionary_string(abort_last_error,disp_string1,slen1);
							DELAY(2,disp_string1);
							process_abort();
						}
						get_dictionary_string(zero_the_input_and_try_again,disp_string1,slen1);
						get_dictionary_string(yes_no,disp_string2,slen2);
						retry=SELECT_FROM_LIST(disp_string1,disp_string2);
						if ( !retry) {
							if ( status[STATUS_RESPONSE_CODE] == 2 ) {
								do {
									tv = ivar_value(transmitter_variable_code);
									get_dictionary_string(select_fld_dev_var_snsr_trim,disp_string1,slen1);
									GET_DEV_VAR_VALUE(disp_string1,transmitter_variable_code);
									if (tv == ivar_value(transmitter_variable_code)) {
										get_dictionary_string(invalid_value_try_again,disp_string1,slen1);
										DELAY(2,disp_string1);
									}
								} while (tv == ivar_value(transmitter_variable_code));
							}
						}
					}
					else {
						get_dictionary_string(snsr_zero_succeeded,disp_string1,slen1);
						DELAY(3,disp_string1);
						process_abort();
					}
				} while ( !retry);
				process_abort();
			}
		}

		REDEFINE METHOD leave_analog_fixed_current_mode
		{
			LABEL [leaving_fixed_current_mode];

			HELP [leaving_fixed_current_mode_help_1];

			DEFINITION
			{
				char status[STATUS_SIZE];
				long var_id[1];
				char disp_string1[60];
				int slen1;

				slen1 =60;

				assign_float(dynamic_variables[ivar_value(analog_output_code)].ANALOG_VALUE,NaN);

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				do {
					get_dictionary_string(return_fld_dev_to_orig_otpt,disp_string1,slen1);
					DELAY(3,disp_string1);
					send(66,status);
					if ( status[STATUS_RESPONSE_CODE] ) {
						display_response_status(66,status[STATUS_RESPONSE_CODE]);
					}
				} while( status[STATUS_RESPONSE_CODE] );
				get_dictionary_string(loop_may_be_returned_to_auto,disp_string1,slen1);
				ACKNOWLEDGE(disp_string1);
			}
		}

		REDEFINE METHOD burn_eeprom
		{
			LABEL [burn_eeprom];

			HELP [burn_eeprom_help];

			DEFINITION
			{
				char status[3];
				char disp_string1[60];
				int slen1;

				slen1 =60;

				IGNORE_ALL_RESPONSE_CODES();
				XMTR_IGNORE_ALL_RESPONSE_CODES();
				IGNORE_ALL_DEVICE_STATUS();
				XMTR_IGNORE_ALL_DEVICE_STATUS();

				assign_int(eeprom_select,0);
				get_dictionary_string(making_changes_permanent,disp_string1,slen1);
				DELAY(3,disp_string1);
				send(39,status);
				if ( status[STATUS_RESPONSE_CODE] ) {
					   display_response_status(39,status[STATUS_RESPONSE_CODE]);
						get_dictionary_string(previous_err_seen_brn_eeprom,disp_string1,slen1);
					   ACKNOWLEDGE(disp_string1);
				}
			}
		}
	}
}
